<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/29/16
 * Time: 8:03 AM
 */

namespace qfq;

require_once(__DIR__ . '/../Constants.php');
require_once(__DIR__ . '/../store/Store.php');
require_once(__DIR__ . '/../store/T3Info.php');
require_once(__DIR__ . '/../store/Store.php');
require_once(__DIR__ . '/../report/Link.php');
require_once(__DIR__ . '/../database/Database.php');
require_once(__DIR__ . '/UserFormException.php');


class AbstractException extends \Exception {

    public $messageArray = array();
    public $store = null;

    protected $file = '';
    protected $line = '';

    /**
     * @return string
     */
    public function formatException() {
        $t3Vars = array();
        $htmlForm = '';

        $debug = '';
        try {
            // In a very early stage, it might be possible that Store can't be initialized: take care not to use it.
            $store = Store::getInstance();
            $t3Vars = $store->getStore(STORE_TYPO3);
        } catch (UserFormException $e) {
            $store = null;
        } catch (CodeException $e) {
            $store = null;
        }

        if (empty($t3Vars)) {
            $t3Vars = T3Info::getVars();
        }

        $html = '';

        $this->messageArray['File'] = $this->getFile();
        $this->messageArray['Line'] = $this->getLine();
        $this->messageArray['Message'] = $this->getMessage();
        $this->messageArray['Code'] = $this->getCode();
        $this->messageArray['Timestamp'] = date('Y.m.d H:i:s O');

        //TODO: 'Client' will remove any HTML Tags before displaying. Formatting needs to be solved together with 'Client',
        // Respect mode: 'form load' (plain HTML), 'Ajax alert'
        $this->messageArray['Stacktrace'] = $this->getTraceAsString();
//        $this->messageArray['Stacktrace'] = $this->getTrace();

        $html .= "Code: " . htmlspecialchars($this->messageArray['Code']) . "<br>";
        $html .= "Message: " . Support::wrapTag("<strong>", htmlspecialchars($this->messageArray['Message'])) . "<br>";

        $html .= 'Page Id: ' . $t3Vars[TYPO3_PAGE_ID] . '<br>';
        $html .= 'Content Id: ' . $t3Vars[TYPO3_TT_CONTENT_UID] . '<br>';

        // Form
        if (isset($this->messageArray['Form']) && $this->messageArray['Form'] != '') {
            $htmlForm = "Form: " . Support::wrapTag("<strong>", htmlspecialchars($this->messageArray['Form'])) . "<br>";
        }

        if (isset($this->messageArray['Form Element']) && $this->messageArray['Form Element'] != '') {
            $html .= "Form Element: " . Support::wrapTag("<strong>", htmlspecialchars($this->messageArray['Form Element'])) . "<br>";
        }

        // Report
        if (isset($this->messageArray[SYSTEM_REPORT_COLUMN_INDEX])) {
            $html .= "Column index: " . Support::wrapTag("<strong>", htmlspecialchars($this->messageArray[SYSTEM_REPORT_COLUMN_INDEX])) . "<br>";
        }

        if (isset($this->messageArray[SYSTEM_REPORT_COLUMN_NAME])) {
            $html .= "Column name: " . Support::wrapTag("<strong>", htmlspecialchars($this->messageArray[SYSTEM_REPORT_COLUMN_NAME])) . "<br>";
        }

        if (isset($this->messageArray[SYSTEM_REPORT_COLUMN_VALUE])) {
            $html .= "Column value: " . Support::wrapTag("<strong>", htmlspecialchars($this->messageArray[SYSTEM_REPORT_COLUMN_VALUE])) . "<br>";
        }

        if ($store !== null && Support::findInSet(SYSTEM_SHOW_DEBUG_INFO_YES, $store->getVar(SYSTEM_SHOW_DEBUG_INFO, STORE_SYSTEM))) {

            $storeSystem = $store->getStore(STORE_SYSTEM);
            $this->messageArray['current sip'] = $store->getStore(STORE_SIP);

            // In case there is a 'form' name given in SIP, we probably have a problem in a form and a direct link to edit the broken form is helpful.
            if (!empty($storeSystem[SYSTEM_FORM])) {
                $renderedUrl = $this->buildFormLink($storeSystem);
                $htmlForm = "Form: " . $renderedUrl . "<br>";
            }

            // Layout
            $debug = '<tr bgcolor="#dddddd"><td colspan="2">Exception</td></tr>';
            foreach ($this->messageArray as $key => $value) {
                // Array to string
                if (is_array($value)) {
                    $value = var_export($value, true);
                }

                if (!empty($value)) {
                    $debug .= "<tr>" . "<td>$key</td>" . "<td>" . htmlspecialchars($value) . "</td>" . "</tr>";
                }
            }

            $debug = '<table border=1 class="qfq-table-100">' . $debug . "</table>";
        }

        $html = "<h2>Error</h2>" . Support::wrapTag('<p>', $html . $htmlForm);
        $html = Support::wrapTag("<div class='warning'>", $html);

        return $html . $debug;
    }

    /**
     * Build a FormEditor link to the broken form.
     */
    private function buildFormLink($storeSystem) {

        $linkForm = '';
        $linkFormElement = '';
        try {
            $db = new Database();
            $sql = "SELECT id FROM Form WHERE name='" . $storeSystem[SYSTEM_FORM] . "'";

            $r = $db->sql($sql, ROW_EXPECT_0_1);
            if (!is_numeric($r[F_ID])) {
                return '';
            }

            $sip = new Sip();
            $link = new Link($sip);

            // Link to 'Form'
            $linkForm = $link->renderLink(TOKEN_SIP . '|' . TOKEN_PAGE . ':' . $storeSystem[SYSTEM_EDIT_FORM_PAGE] . '&' . CLIENT_FORM . '=' . FORM_NAME_FORM . '&' .
                CLIENT_RECORD_ID . '=' . $r[F_ID] . '|' . TOKEN_TEXT . ':' . $storeSystem[SYSTEM_FORM]);

            // Link to 'FormElement'
            if (!empty($storeSystem[SYSTEM_FORM_ELEMENT_ID])) {
                $linkFormElement = ' / ' . $link->renderLink(TOKEN_SIP . '|' . TOKEN_PAGE . ':' . $storeSystem[SYSTEM_EDIT_FORM_PAGE] . '&' . CLIENT_FORM . '=' . FORM_NAME_FORM_ELEMENT . '&' .
                        CLIENT_RECORD_ID . '=' . $storeSystem[SYSTEM_FORM_ELEMENT_ID] . '|' . TOKEN_TEXT . ':' . 'FormElement: ' . $storeSystem[SYSTEM_FORM_ELEMENT_ID]);
            }

        } catch (\exception $e) {
        }

        return $linkForm . $linkFormElement;
    }
}